Aws.add_service(:Route53Domains, {
  api: "#{Aws::API_DIR}/route53domains/2014-05-15/api-2.json",
  docs: "#{Aws::API_DIR}/route53domains/2014-05-15/docs-2.json",
  examples: "#{Aws::API_DIR}/route53domains/2014-05-15/examples-1.json",
  paginators: "#{Aws::API_DIR}/route53domains/2014-05-15/paginators-1.json",
})
