Aws.add_service(:ResourceGroupsTaggingAPI, {
  api: "#{Aws::API_DIR}/resourcegroupstaggingapi/2017-01-26/api-2.json",
  docs: "#{Aws::API_DIR}/resourcegroupstaggingapi/2017-01-26/docs-2.json",
  examples: "#{Aws::API_DIR}/resourcegroupstaggingapi/2017-01-26/examples-1.json",
  paginators: "#{Aws::API_DIR}/resourcegroupstaggingapi/2017-01-26/paginators-1.json",
})
