Aws.add_service(:STS, {
  api: "#{Aws::API_DIR}/sts/2011-06-15/api-2.json",
  docs: "#{Aws::API_DIR}/sts/2011-06-15/docs-2.json",
  examples: "#{Aws::API_DIR}/sts/2011-06-15/examples-1.json",
  paginators: "#{Aws::API_DIR}/sts/2011-06-15/paginators-1.json",
})
