Aws.add_service(:Shield, {
  api: "#{Aws::API_DIR}/shield/2016-06-02/api-2.json",
  docs: "#{Aws::API_DIR}/shield/2016-06-02/docs-2.json",
  examples: "#{Aws::API_DIR}/shield/2016-06-02/examples-1.json",
})
