Aws.add_service(:Rekognition, {
  api: "#{Aws::API_DIR}/rekognition/2016-06-27/api-2.json",
  docs: "#{Aws::API_DIR}/rekognition/2016-06-27/docs-2.json",
  examples: "#{Aws::API_DIR}/rekognition/2016-06-27/examples-1.json",
  paginators: "#{Aws::API_DIR}/rekognition/2016-06-27/paginators-1.json",
})
