Aws.add_service(:Health, {
  api: "#{Aws::API_DIR}/health/2016-08-04/api-2.json",
  docs: "#{Aws::API_DIR}/health/2016-08-04/docs-2.json",
  examples: "#{Aws::API_DIR}/health/2016-08-04/examples-1.json",
  paginators: "#{Aws::API_DIR}/health/2016-08-04/paginators-1.json",
})
