Aws.add_service(:SSM, {
  api: "#{Aws::API_DIR}/ssm/2014-11-06/api-2.json",
  docs: "#{Aws::API_DIR}/ssm/2014-11-06/docs-2.json",
  examples: "#{Aws::API_DIR}/ssm/2014-11-06/examples-1.json",
  paginators: "#{Aws::API_DIR}/ssm/2014-11-06/paginators-1.json",
})
