Aws.add_service(:CloudWatchLogs, {
  api: "#{Aws::API_DIR}/logs/2014-03-28/api-2.json",
  docs: "#{Aws::API_DIR}/logs/2014-03-28/docs-2.json",
  examples: "#{Aws::API_DIR}/logs/2014-03-28/examples-1.json",
  paginators: "#{Aws::API_DIR}/logs/2014-03-28/paginators-1.json",
})
