Aws.add_service(:MTurk, {
  api: "#{Aws::API_DIR}/mturk-requester/2017-01-17/api-2.json",
  docs: "#{Aws::API_DIR}/mturk-requester/2017-01-17/docs-2.json",
  examples: "#{Aws::API_DIR}/mturk-requester/2017-01-17/examples-1.json",
  paginators: "#{Aws::API_DIR}/mturk-requester/2017-01-17/paginators-1.json",
})
