Aws.add_service(:LambdaPreview, {
  api: "#{Aws::API_DIR}/lambda/2014-11-11/api-2.json",
  docs: "#{Aws::API_DIR}/lambda/2014-11-11/docs-2.json",
  paginators: "#{Aws::API_DIR}/lambda/2014-11-11/paginators-1.json",
})
