Aws.add_service(:MachineLearning, {
  api: "#{Aws::API_DIR}/machinelearning/2014-12-12/api-2.json",
  docs: "#{Aws::API_DIR}/machinelearning/2014-12-12/docs-2.json",
  examples: "#{Aws::API_DIR}/machinelearning/2014-12-12/examples-1.json",
  paginators: "#{Aws::API_DIR}/machinelearning/2014-12-12/paginators-1.json",
  waiters: "#{Aws::API_DIR}/machinelearning/2014-12-12/waiters-2.json",
})
