Aws.add_service(:KinesisAnalytics, {
  api: "#{Aws::API_DIR}/kinesisanalytics/2015-08-14/api-2.json",
  docs: "#{Aws::API_DIR}/kinesisanalytics/2015-08-14/docs-2.json",
  examples: "#{Aws::API_DIR}/kinesisanalytics/2015-08-14/examples-1.json",
  paginators: "#{Aws::API_DIR}/kinesisanalytics/2015-08-14/paginators-1.json",
})
