Aws.add_service(:SMS, {
  api: "#{Aws::API_DIR}/sms/2016-10-24/api-2.json",
  docs: "#{Aws::API_DIR}/sms/2016-10-24/docs-2.json",
  examples: "#{Aws::API_DIR}/sms/2016-10-24/examples-1.json",
  paginators: "#{Aws::API_DIR}/sms/2016-10-24/paginators-1.json",
})
