Aws.add_service(:Inspector, {
  api: "#{Aws::API_DIR}/inspector/2016-02-16/api-2.json",
  docs: "#{Aws::API_DIR}/inspector/2016-02-16/docs-2.json",
  examples: "#{Aws::API_DIR}/inspector/2016-02-16/examples-1.json",
  paginators: "#{Aws::API_DIR}/inspector/2016-02-16/paginators-1.json",
})
