Aws.add_service(:CloudTrail, {
  api: "#{Aws::API_DIR}/cloudtrail/2013-11-01/api-2.json",
  docs: "#{Aws::API_DIR}/cloudtrail/2013-11-01/docs-2.json",
  examples: "#{Aws::API_DIR}/cloudtrail/2013-11-01/examples-1.json",
  paginators: "#{Aws::API_DIR}/cloudtrail/2013-11-01/paginators-1.json",
})
