Aws.add_service(:StorageGateway, {
  api: "#{Aws::API_DIR}/storagegateway/2013-06-30/api-2.json",
  docs: "#{Aws::API_DIR}/storagegateway/2013-06-30/docs-2.json",
  examples: "#{Aws::API_DIR}/storagegateway/2013-06-30/examples-1.json",
  paginators: "#{Aws::API_DIR}/storagegateway/2013-06-30/paginators-1.json",
})
