Aws.add_service(:Firehose, {
  api: "#{Aws::API_DIR}/firehose/2015-08-04/api-2.json",
  docs: "#{Aws::API_DIR}/firehose/2015-08-04/docs-2.json",
  examples: "#{Aws::API_DIR}/firehose/2015-08-04/examples-1.json",
})
