Aws.add_service(:WAF, {
  api: "#{Aws::API_DIR}/waf/2015-08-24/api-2.json",
  docs: "#{Aws::API_DIR}/waf/2015-08-24/docs-2.json",
  examples: "#{Aws::API_DIR}/waf/2015-08-24/examples-1.json",
  paginators: "#{Aws::API_DIR}/waf/2015-08-24/paginators-1.json",
})
