Aws.add_service(:CloudWatchEvents, {
  api: "#{Aws::API_DIR}/events/2015-10-07/api-2.json",
  docs: "#{Aws::API_DIR}/events/2015-10-07/docs-2.json",
  examples: "#{Aws::API_DIR}/events/2015-10-07/examples-1.json",
  paginators: "#{Aws::API_DIR}/events/2015-10-07/paginators-1.json",
})
