Aws.add_service(:SQS, {
  api: "#{Aws::API_DIR}/sqs/2012-11-05/api-2.json",
  docs: "#{Aws::API_DIR}/sqs/2012-11-05/docs-2.json",
  examples: "#{Aws::API_DIR}/sqs/2012-11-05/examples-1.json",
  paginators: "#{Aws::API_DIR}/sqs/2012-11-05/paginators-1.json",
  resources: "#{Aws::API_DIR}/sqs/2012-11-05/resources-1.json",
})
