Aws.add_service(:DirectConnect, {
  api: "#{Aws::API_DIR}/directconnect/2012-10-25/api-2.json",
  docs: "#{Aws::API_DIR}/directconnect/2012-10-25/docs-2.json",
  examples: "#{Aws::API_DIR}/directconnect/2012-10-25/examples-1.json",
  paginators: "#{Aws::API_DIR}/directconnect/2012-10-25/paginators-1.json",
})
