Aws.add_service(:OpsWorks, {
  api: "#{Aws::API_DIR}/opsworks/2013-02-18/api-2.json",
  docs: "#{Aws::API_DIR}/opsworks/2013-02-18/docs-2.json",
  examples: "#{Aws::API_DIR}/opsworks/2013-02-18/examples-1.json",
  paginators: "#{Aws::API_DIR}/opsworks/2013-02-18/paginators-1.json",
  resources: "#{Aws::API_DIR}/opsworks/2013-02-18/resources-1.json",
  waiters: "#{Aws::API_DIR}/opsworks/2013-02-18/waiters-2.json",
})
