Aws.add_service(:CognitoIdentityProvider, {
  api: "#{Aws::API_DIR}/cognito-idp/2016-04-18/api-2.json",
  docs: "#{Aws::API_DIR}/cognito-idp/2016-04-18/docs-2.json",
  examples: "#{Aws::API_DIR}/cognito-idp/2016-04-18/examples-1.json",
  paginators: "#{Aws::API_DIR}/cognito-idp/2016-04-18/paginators-1.json",
})
