Aws.add_service(:CodePipeline, {
  api: "#{Aws::API_DIR}/codepipeline/2015-07-09/api-2.json",
  docs: "#{Aws::API_DIR}/codepipeline/2015-07-09/docs-2.json",
  examples: "#{Aws::API_DIR}/codepipeline/2015-07-09/examples-1.json",
})
