Aws.add_service(:DirectoryService, {
  api: "#{Aws::API_DIR}/ds/2015-04-16/api-2.json",
  docs: "#{Aws::API_DIR}/ds/2015-04-16/docs-2.json",
  examples: "#{Aws::API_DIR}/ds/2015-04-16/examples-1.json",
})
