Aws.add_service(:Budgets, {
  api: "#{Aws::API_DIR}/budgets/2016-10-20/api-2.json",
  docs: "#{Aws::API_DIR}/budgets/2016-10-20/docs-2.json",
  examples: "#{Aws::API_DIR}/budgets/2016-10-20/examples-1.json",
  paginators: "#{Aws::API_DIR}/budgets/2016-10-20/paginators-1.json",
})
