Aws.add_service(:CloudDirectory, {
  api: "#{Aws::API_DIR}/clouddirectory/2016-05-10/api-2.json",
  docs: "#{Aws::API_DIR}/clouddirectory/2016-05-10/docs-2.json",
  examples: "#{Aws::API_DIR}/clouddirectory/2016-05-10/examples-1.json",
  paginators: "#{Aws::API_DIR}/clouddirectory/2016-05-10/paginators-1.json",
})
