Aws.add_service(:MarketplaceEntitlementService, {
  api: "#{Aws::API_DIR}/entitlement.marketplace/2017-01-11/api-2.json",
  docs: "#{Aws::API_DIR}/entitlement.marketplace/2017-01-11/docs-2.json",
  examples: "#{Aws::API_DIR}/entitlement.marketplace/2017-01-11/examples-1.json",
  paginators: "#{Aws::API_DIR}/entitlement.marketplace/2017-01-11/paginators-1.json",
})
