Aws.add_service(:SNS, {
  api: "#{Aws::API_DIR}/sns/2010-03-31/api-2.json",
  docs: "#{Aws::API_DIR}/sns/2010-03-31/docs-2.json",
  examples: "#{Aws::API_DIR}/sns/2010-03-31/examples-1.json",
  paginators: "#{Aws::API_DIR}/sns/2010-03-31/paginators-1.json",
  resources: "#{Aws::API_DIR}/sns/2010-03-31/resources-1.json",
})
