Aws.add_service(:ECR, {
  api: "#{Aws::API_DIR}/ecr/2015-09-21/api-2.json",
  docs: "#{Aws::API_DIR}/ecr/2015-09-21/docs-2.json",
  examples: "#{Aws::API_DIR}/ecr/2015-09-21/examples-1.json",
  paginators: "#{Aws::API_DIR}/ecr/2015-09-21/paginators-1.json",
})
