Aws.add_service(:ElasticTranscoder, {
  api: "#{Aws::API_DIR}/elastictranscoder/2012-09-25/api-2.json",
  docs: "#{Aws::API_DIR}/elastictranscoder/2012-09-25/docs-2.json",
  examples: "#{Aws::API_DIR}/elastictranscoder/2012-09-25/examples-1.json",
  paginators: "#{Aws::API_DIR}/elastictranscoder/2012-09-25/paginators-1.json",
  waiters: "#{Aws::API_DIR}/elastictranscoder/2012-09-25/waiters-2.json",
})
