Aws.add_service(:DeviceFarm, {
  api: "#{Aws::API_DIR}/devicefarm/2015-06-23/api-2.json",
  docs: "#{Aws::API_DIR}/devicefarm/2015-06-23/docs-2.json",
  examples: "#{Aws::API_DIR}/devicefarm/2015-06-23/examples-1.json",
  paginators: "#{Aws::API_DIR}/devicefarm/2015-06-23/paginators-1.json",
})
