Aws.add_service(:Lex, {
  api: "#{Aws::API_DIR}/runtime.lex/2016-11-28/api-2.json",
  docs: "#{Aws::API_DIR}/runtime.lex/2016-11-28/docs-2.json",
  examples: "#{Aws::API_DIR}/runtime.lex/2016-11-28/examples-1.json",
  paginators: "#{Aws::API_DIR}/runtime.lex/2016-11-28/paginators-1.json",
})
