Aws.add_service(:ServiceCatalog, {
  api: "#{Aws::API_DIR}/servicecatalog/2015-12-10/api-2.json",
  docs: "#{Aws::API_DIR}/servicecatalog/2015-12-10/docs-2.json",
  examples: "#{Aws::API_DIR}/servicecatalog/2015-12-10/examples-1.json",
})
