Aws.add_service(:LexModelBuildingService, {
  api: "#{Aws::API_DIR}/lex-models/2017-04-19/api-2.json",
  docs: "#{Aws::API_DIR}/lex-models/2017-04-19/docs-2.json",
  examples: "#{Aws::API_DIR}/lex-models/2017-04-19/examples-1.json",
  paginators: "#{Aws::API_DIR}/lex-models/2017-04-19/paginators-1.json",
})
