Aws.add_service(:DynamoDBStreams, {
  api: "#{Aws::API_DIR}/streams.dynamodb/2012-08-10/api-2.json",
  docs: "#{Aws::API_DIR}/streams.dynamodb/2012-08-10/docs-2.json",
  examples: "#{Aws::API_DIR}/streams.dynamodb/2012-08-10/examples-1.json",
  paginators: "#{Aws::API_DIR}/streams.dynamodb/2012-08-10/paginators-1.json",
})
