Aws.add_service(:ECS, {
  api: "#{Aws::API_DIR}/ecs/2014-11-13/api-2.json",
  docs: "#{Aws::API_DIR}/ecs/2014-11-13/docs-2.json",
  examples: "#{Aws::API_DIR}/ecs/2014-11-13/examples-1.json",
  paginators: "#{Aws::API_DIR}/ecs/2014-11-13/paginators-1.json",
  waiters: "#{Aws::API_DIR}/ecs/2014-11-13/waiters-2.json",
})
