Aws.add_service(:SimpleDB, {
  api: "#{Aws::API_DIR}/sdb/2009-04-15/api-2.json",
  docs: "#{Aws::API_DIR}/sdb/2009-04-15/docs-2.json",
  paginators: "#{Aws::API_DIR}/sdb/2009-04-15/paginators-1.json",
})
