Aws.add_service(:APIGateway, {
  api: "#{Aws::API_DIR}/apigateway/2015-07-09/api-2.json",
  docs: "#{Aws::API_DIR}/apigateway/2015-07-09/docs-2.json",
  examples: "#{Aws::API_DIR}/apigateway/2015-07-09/examples-1.json",
  paginators: "#{Aws::API_DIR}/apigateway/2015-07-09/paginators-1.json",
})
