Aws.add_service(:States, {
  api: "#{Aws::API_DIR}/states/2016-11-23/api-2.json",
  docs: "#{Aws::API_DIR}/states/2016-11-23/docs-2.json",
  examples: "#{Aws::API_DIR}/states/2016-11-23/examples-1.json",
  paginators: "#{Aws::API_DIR}/states/2016-11-23/paginators-1.json",
})
