Aws.add_service(:IAM, {
  api: "#{Aws::API_DIR}/iam/2010-05-08/api-2.json",
  docs: "#{Aws::API_DIR}/iam/2010-05-08/docs-2.json",
  examples: "#{Aws::API_DIR}/iam/2010-05-08/examples-1.json",
  paginators: "#{Aws::API_DIR}/iam/2010-05-08/paginators-1.json",
  resources: "#{Aws::API_DIR}/iam/2010-05-08/resources-1.json",
  waiters: "#{Aws::API_DIR}/iam/2010-05-08/waiters-2.json",
})
