Aws.add_service(:WorkSpaces, {
  api: "#{Aws::API_DIR}/workspaces/2015-04-08/api-2.json",
  docs: "#{Aws::API_DIR}/workspaces/2015-04-08/docs-2.json",
  examples: "#{Aws::API_DIR}/workspaces/2015-04-08/examples-1.json",
  paginators: "#{Aws::API_DIR}/workspaces/2015-04-08/paginators-1.json",
})
