Aws.add_service(:CodeBuild, {
  api: "#{Aws::API_DIR}/codebuild/2016-10-06/api-2.json",
  docs: "#{Aws::API_DIR}/codebuild/2016-10-06/docs-2.json",
  examples: "#{Aws::API_DIR}/codebuild/2016-10-06/examples-1.json",
  paginators: "#{Aws::API_DIR}/codebuild/2016-10-06/paginators-1.json",
})
