Aws.add_service(:CloudHSM, {
  api: "#{Aws::API_DIR}/cloudhsm/2014-05-30/api-2.json",
  docs: "#{Aws::API_DIR}/cloudhsm/2014-05-30/docs-2.json",
  examples: "#{Aws::API_DIR}/cloudhsm/2014-05-30/examples-1.json",
})
