Aws.add_service(:Batch, {
  api: "#{Aws::API_DIR}/batch/2016-08-10/api-2.json",
  docs: "#{Aws::API_DIR}/batch/2016-08-10/docs-2.json",
  examples: "#{Aws::API_DIR}/batch/2016-08-10/examples-1.json",
  paginators: "#{Aws::API_DIR}/batch/2016-08-10/paginators-1.json",
})
